#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#define COUNTOF(x)  (sizeof(x)/sizeof(x[0]))


void TrackMouse(HWND hwnd, BOOL nonClient = FALSE)
{
    TRACKMOUSEEVENT tme;
    tme.cbSize = sizeof(TRACKMOUSEEVENT);
    tme.dwFlags = TME_HOVER | TME_LEAVE | (nonClient ? TME_NONCLIENT : 0); //Type of events to track & trigger.
    tme.dwHoverTime = 1; //How long the mouse has to be in the window to trigger a hover event.
    tme.hwndTrack = hwnd;
    TrackMouseEvent(&tme);
}

TCHAR wndText[1024] = {0};

void SetText(HWND hwnd, LPCTSTR text)
{
	TCHAR wndTextLocal[1024] = {0};
	_tcsncpy(wndTextLocal, wndText, COUNTOF(wndTextLocal) - 1);
	_tcsncpy(wndText, text, COUNTOF(wndText) - 1); 
	_tcscat(wndText, TEXT("\r\n"));
	size_t len = _tcslen(wndText);
	size_t pos = COUNTOF(wndTextLocal) - 1 - len;
	if (pos > 0)
		wndTextLocal[pos] = TEXT('\0');
	_tcscat(wndText, wndTextLocal);
	InvalidateRect(hwnd, NULL, TRUE);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static bool Tracking = false;
    static UINT msgPrev  = WM_NULL;
    static BOOL nonClient = FALSE;

    switch(msg)
    {
        case WM_PAINT:
        {
        	RECT rc = {0};
        	GetClientRect(hwnd, &rc);
        	PAINTSTRUCT ps = {0};
        	BeginPaint(hwnd, &ps);
        	DrawText(ps.hdc, wndText, _tcslen(wndText), &rc, DT_TOP); 
        	EndPaint(hwnd, &ps);
        	break;
        }
        
        case WM_CLOSE:
            DestroyWindow(hwnd);
        break;
        case WM_KEYUP:
            if (wParam ==VK_ESCAPE)
                DestroyWindow(hwnd);
            else
            {
                char strDataToSend[32];
                sprintf(strDataToSend, "%c", wParam);
                MessageBox(NULL,strDataToSend, "keyselected",MB_OK);
            }
        break;

        case WM_NCMOUSEMOVE:
        	nonClient = TRUE;
        case WM_MOUSEMOVE:
            TrackMouse(hwnd, nonClient);
            return 0;
        case WM_NCMOUSEHOVER:
          if (msg != msgPrev)
            SetText(hwnd,"MOUSE hover NC");
            break;
        case WM_NCMOUSELEAVE:
          if (msg != msgPrev)
            SetText(hwnd,"MOUSE leave NC");
            nonClient = FALSE;
            break;
        case WM_MOUSEHOVER:
          if (msg != msgPrev)
            SetText(hwnd,"MOUSE hover");
            break;
        case WM_MOUSELEAVE:
          if (msg != msgPrev)
            SetText(hwnd,"MOUSE leave");
            break;
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }

    if (msg == WM_NCMOUSEHOVER || 
        msg == WM_NCMOUSELEAVE ||
        msg == WM_MOUSEHOVER || 
        msg == WM_MOUSELEAVE)
	    msgPrev = msg;

    return 0;
}

int _tWinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmd, INT nShow)
{
	WNDCLASS wc = {0};
	wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wc.lpfnWndProc = WndProc;
	wc.hInstance   = hInst;
	wc.hCursor     = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wc.lpszClassName = TEXT("MOUSEWND");

	ATOM wca = RegisterClass(&wc);
	if (!wca)
		return 1;

	HWND hwnd = CreateWindow(wc.lpszClassName, TEXT("MOUSE"), WS_OVERLAPPEDWINDOW, 
								CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
								NULL, (HMENU)NULL, hInst, (LPVOID)NULL);
	ShowWindow(hwnd, SW_SHOWNORMAL);
	UpdateWindow(hwnd);

	MSG msg;

	while (0 < GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}